# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
# Copyright (C) 2022 RERO.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Permissions for Invenio-Admin."""

import importlib_metadata
from flask_principal import ActionNeed

action_admin_access = ActionNeed("admin-access")
"""Define the action needed by the default permission factory."""


def admin_permission_factory(admin_view):
    """Default factory for creating a permission for an admin.

    It tries to load a :class:`invenio_access.permissions.Permission`
    instance if `invenio_access` is installed.
    Otherwise, it loads a :class:`flask_principal.Permission` instance.

    :param admin_view: Instance of administration view which is currently being
        protected.
    :returns: Permission instance.
    """
    try:
        importlib_metadata.version("invenio-access")
        from invenio_access import Permission
    except importlib_metadata.PackageNotFoundError:
        from flask_principal import Permission

    return Permission(action_admin_access)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Mocks of DB Models and Admin's ModelViews for entrypoint testing."""

from flask_admin.contrib.sqla import ModelView
from invenio_db import db


class ModelThree(db.Model):
    """Test model with just one column."""

    id = db.Column(db.Integer, primary_key=True)
    """Id of the model."""


class ModelThreeModelView(ModelView):
    """AdminModelView of the ModelOne."""

    pass


three = dict(modelview=ModelThreeModelView, model=ModelThree)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
# Copyright (C) 2023-2024 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.


"""Pytest configuration."""

from __future__ import absolute_import, print_function

import shutil
import tempfile
import uuid

import pytest
from flask import Flask
from flask_admin.base import BaseView, expose
from flask_admin.contrib.sqla import ModelView
from flask_login import LoginManager, UserMixin, current_user, login_user
from flask_menu import Menu
from flask_principal import (
    Identity,
    Permission,
    Principal,
    UserNeed,
    identity_changed,
    identity_loaded,
)
from invenio_access import InvenioAccess
from invenio_db import InvenioDB, db
from invenio_i18n import Babel
from sqlalchemy.dialects import mysql
from sqlalchemy_utils.functions import create_database, database_exists, drop_database
from sqlalchemy_utils.types import UUIDType

from invenio_admin import InvenioAdmin
from invenio_admin.ext import finalize_app
from invenio_admin.permissions import action_admin_access
from invenio_admin.views import blueprint


class TestModel(db.Model):
    """Test model with just one column."""

    id = db.Column(db.Integer, primary_key=True)
    """Id of the model."""

    uuidcol = db.Column(UUIDType, default=uuid.uuid4)
    """UUID test column."""

    dt = db.Column(
        db.DateTime().with_variant(mysql.DATETIME(), "mysql"),
        nullable=True,
    )


class TestModelView(ModelView):
    """AdminModelView of the TestModel."""


class TestBase(BaseView):
    """Base AdminView."""

    @expose("/")
    def index(self):
        """Index page."""
        return "HelloWorld"

    @expose("/foo/")
    def foo(self):
        """Another page."""
        return "Foobar!"


class TestUser(UserMixin):
    """Test user class."""

    def __init__(self, user_id):
        """Constructor of the user."""
        self.id = int(user_id)

    @classmethod
    def get(cls, user_id):
        """Getter of the TestUser."""
        return cls(user_id)


@pytest.fixture()
def testmodelcls():
    """Get the test model class."""
    return TestModel


@pytest.fixture()
def app(request):
    """Flask application fixture."""
    instance_path = tempfile.mkdtemp()
    app = Flask("testapp", instance_path=instance_path)
    app.config.update(
        TESTING=True,
        SECRET_KEY="SECRET_KEY",
        ADMIN_LOGIN_ENDPOINT="login",
        SQLALCHEMY_TRACK_MODIFICATIONS=False,
        APP_THEME=[],
        THEME_ICONS=[],
        DB_VERSIONING=False,
    )
    Babel(app)
    InvenioDB(app)
    InvenioAccess(app)
    Principal(app)
    LoginManager(app)
    Menu(app)

    # Install login and access loading.
    @app.login_manager.user_loader
    def load_user(user_id):
        return TestUser.get(user_id)

    @app.route("/login/")
    def login():
        from flask import current_app
        from flask import request as flask_request

        user = TestUser.get(flask_request.args.get("user", 1))
        login_user(user)
        identity_changed.send(
            current_app._get_current_object(), identity=Identity(user.id)
        )
        return "Logged In"

    @identity_loaded.connect_via(app)
    def on_identity_loaded(sender, identity):
        identity.user = current_user
        identity.provides.add(UserNeed(current_user.id))
        if current_user.id == 1:
            identity.provides.add(action_admin_access)

    # Register admin view
    InvenioAdmin(app, permission_factory=lambda x: Permission(action_admin_access))
    app.extensions["invenio-admin"].register_view(TestModelView, TestModel, db.session)
    app.extensions["invenio-admin"].register_view(TestBase)
    app.register_blueprint(blueprint)

    with app.app_context():
        finalize_app(app)

    # Create database
    with app.app_context():
        if not database_exists(str(db.engine.url)):
            create_database(str(db.engine.url))
        db.drop_all()
        db.create_all()

    def teardown():
        with app.app_context():
            drop_database(str(db.engine.url))
        shutil.rmtree(instance_path)

    request.addfinalizer(teardown)
    return app



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Filters module tests."""

from __future__ import absolute_import, print_function

import uuid

from invenio_admin.filters import FilterConverter, UUIDEqualFilter


def test_uuid_filter(app, testmodelcls):
    """Test UUID."""
    with app.app_context():
        f = UUIDEqualFilter(testmodelcls.uuidcol, "uuidcol")
        q = testmodelcls.query
        assert q.whereclause is None

        q_applied = f.apply(testmodelcls.query, str(uuid.uuid4()), None)
        assert q_applied.whereclause is not None

        q_applied = f.apply(testmodelcls.query, "", None)
        assert q_applied.whereclause is None

        q_applied = f.apply(testmodelcls.query, "test", None)
        assert q_applied.whereclause is None


def test_filter_converter_uuid(testmodelcls):
    """Test filter converter."""
    c = FilterConverter()
    f = c.convert("uuidtype", testmodelcls.uuidcol, "uuidcol")
    assert len(f) == 1
    assert isinstance(f[0], UUIDEqualFilter)


def test_filter_converter_variant(testmodelcls):
    """Test filter converter."""
    c = FilterConverter()
    f = c.convert("variant", testmodelcls.dt, "dt")
    assert len(f) == 7



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Configuration for Invenio-Admin."""

ADMIN_BASE_TEMPLATE = None
"""Admin panel base template.
By default (``None``) uses the Flask-Admin template."""

ADMIN_APPNAME = "Invenio"
"""Name of the Flask-Admin app (also the page title of admin panel)."""

ADMIN_LOGIN_ENDPOINT = "security.login"
"""Endpoint name of the login view. Anonymous users trying to access admin
panel will be redirected to this endpoint."""

ADMIN_LOGOUT_ENDPOINT = "security.logout"
"""Endpoint name of logout view."""

ADMIN_TEMPLATE_MODE = "bootstrap3"
"""Flask-Admin template mode. Either ``bootstrap2`` or ``bootstrap3``."""

ADMIN_PERMISSION_FACTORY = "invenio_admin.permissions.admin_permission_factory"
"""Permission factory for the admin views."""



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
# Copyright (C) 2022 RERO.
# Copyright (C) 2024 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.


"""Module tests."""

import importlib
from unittest.mock import patch

import flask_admin
import importlib_metadata
import pytest
from flask import Flask
from flask_admin.contrib.sqla import ModelView
from flask_menu import current_menu
from importlib_metadata import EntryPoint
from invenio_access.permissions import Permission
from invenio_db import db
from invenio_theme import InvenioTheme

from invenio_admin import InvenioAdmin
from invenio_admin.permissions import admin_permission_factory
from invenio_admin.views import protected_adminview_factory


def test_version():
    """Test version import."""
    from invenio_admin import __version__

    assert __version__


def test_init():
    """Test extension initialization."""
    app = Flask("testapp")
    ext = InvenioAdmin(app)
    assert "invenio-admin" in app.extensions

    app = Flask("testapp")
    ext = InvenioAdmin()
    assert "invenio-admin" not in app.extensions
    ext.init_app(app)
    assert "invenio-admin" in app.extensions


def test_invenio_theme_loading_order():
    """Test base template set correctly by invenio_theme."""
    app = Flask("testapp")
    InvenioAdmin(app)
    assert app.config.get("ADMIN_BASE_TEMPLATE") is None

    app = Flask("testapp")
    InvenioTheme(app)
    InvenioAdmin(app)
    assert app.config.get("ADMIN_BASE_TEMPLATE") is not None

    app = Flask("testapp")
    InvenioAdmin(app)
    InvenioTheme(app)
    assert app.config.get("ADMIN_BASE_TEMPLATE") is not None


def test_base_template_override():
    """Test base template is set up correctly."""
    app = Flask("testapp")
    base_template = "test_base_template.html"
    app.config["ADMIN_BASE_TEMPLATE"] = base_template
    InvenioTheme(app)
    InvenioAdmin(app)
    # state = InvenioAdmin(app)
    assert app.config.get("ADMIN_BASE_TEMPLATE") == base_template

    # TODO: this check can't work, because of the removed
    # try_trigger_before_first_request_functions from flask. There has to be
    # found another way to test that
    # # Force call of before_first_request registered triggers.
    # app.try_trigger_before_first_request_functions()
    # assert state.admin.base_template == base_template


def test_default_permission(app):
    """Test loading of default permission class."""
    with app.app_context():
        with patch("importlib_metadata.version") as get_distribution:
            get_distribution.side_effect = importlib_metadata.PackageNotFoundError
            assert not isinstance(admin_permission_factory(None), Permission)

    assert isinstance(admin_permission_factory(None), Permission)


def test_admin_view_authenticated(app):
    """Test the authentication for the admin."""
    with app.test_client() as client:
        res = client.get("/admin/", follow_redirects=False)
        # Assert redirect to login
        assert res.status_code == 302

    # Unauthenticated users are redirect to login page.
    with app.test_client() as client:
        # Model view
        res = client.get("/admin/testmodel/", follow_redirects=False)
        assert res.status_code == 302
        assert res.location.startswith("/login")

        # Base view
        res = client.get("/admin/testbase/")
        assert res.status_code == 302
        assert res.location.startswith("/login")
        res = client.get("/admin/testbase/foo/")
        assert res.status_code == 302
        assert res.location.startswith("/login")

    # User 1 can access admin because it has ActioNeed(admin-access)
    with app.test_client() as client:
        res = client.get("/login/?user=1")
        assert res.status_code == 200
        # Admin panel
        res = client.get("/admin/")
        assert res.status_code == 200

        # Model view
        res = client.get("/admin/testmodel/")
        assert res.status_code == 200

        # Base view
        res = client.get("/admin/testbase/")
        assert res.status_code == 200
        res = client.get("/admin/testbase/foo/")
        assert res.status_code == 200

    # User 2 is missing ActioNeed(admin-access) and thus can't access admin.
    # 403 error returned.
    with app.test_client() as client:
        res = client.get("/login/?user=2")
        # Admin panel
        res = client.get("/admin/")
        assert res.status_code == 403

        # Model view
        res = client.get("/admin/testmodel/")
        assert res.status_code == 403

        # Base view
        res = client.get("/admin/testbase/")
        assert res.status_code == 403
        res = client.get("/admin/testbase/foo/")
        assert res.status_code == 403


def test_menu_visiblity(app):
    """Test menu visiblity."""

    @app.route("/menu/")
    def render_menu():
        settings_menu = current_menu.submenu("settings")
        return ";".join([i.url for i in settings_menu.children if i.visible])

    with app.test_client() as client:
        res = client.get("/menu/")
        assert "/admin/" not in res.get_data(as_text=True)
        res = client.get("/login/?user=1")
        assert res.status_code == 200
        res = client.get("/menu/")
        assert "/admin/" in res.get_data(as_text=True)


def test_custom_permissions(app, testmodelcls):
    """Test custom permissions."""

    class CustomModel(testmodelcls):
        """Some custom model."""

        pass

    class CustomView(ModelView):
        """Some custom model."""

        @staticmethod
        def is_accessible():
            """Check if accessible."""
            return False

    protected_view = protected_adminview_factory(CustomView)
    app.extensions["admin"][0].add_view(protected_view(CustomModel, db.session))

    with app.test_client() as client:
        res = client.get("/login/?user=1")
        assert res.status_code == 200
        res = client.get("/admin/")
        assert res.status_code == 200
        res = client.get("/admin/testmodel/")
        assert res.status_code == 200
        res = client.get("/admin/testbase/")
        assert res.status_code == 200
        res = client.get("/admin/custommodel/")
        assert res.status_code == 403


class MockEntryPoint(EntryPoint):
    """Mock of EntryPoint."""

    def load(self):
        """Mock the load of entry point."""
        mod = importlib.import_module(self.module)
        obj = getattr(mod, self.name)
        return obj


def _mock_iter_entry_points():
    """Mock the entry point iterator."""

    def fn(group=None):
        entry_points = {
            "invenio_admin.views_invalid": [
                MockEntryPoint(
                    name="zero",
                    value="demo.onetwo",
                    group="invenio_admin.views_invalid",
                ),
            ],
            "invenio_admin.views": [
                MockEntryPoint(
                    name="one", value="demo.onetwo", group="invenio_admin.views"
                ),
                MockEntryPoint(
                    name="two", value="demo.onetwo", group="invenio_admin.views"
                ),
                MockEntryPoint(
                    name="three", value="demo.three", group="invenio_admin.views"
                ),
                MockEntryPoint(
                    name="four", value="demo.four", group="invenio_admin.views"
                ),
            ],
        }
        if group:
            entry_points_group = []
            for eps in entry_points.values():
                for ep in eps:
                    if ep.group == group:
                        entry_points_group.append(ep)
            return entry_points_group
        return entry_points

    return fn


@patch("importlib_metadata.entry_points", _mock_iter_entry_points())
def test_invalid_entry_points():
    """Test invalid admin views discovery through entry points."""
    app = Flask("testapp")
    admin_app = InvenioAdmin()
    with pytest.raises(Exception) as e:
        admin_app.init_app(app, entry_point_group="invenio_admin.views_invalid")
    assert '"view_class"' in str(e)


@patch("importlib_metadata.entry_points", _mock_iter_entry_points())
def test_entry_points():
    """Test admin views discovery through entry points."""
    from flask_principal import Permission

    app = Flask("testapp")
    admin_app = InvenioAdmin(
        app, permission_factory=lambda x: Permission(), view_class_factory=lambda x: x
    )
    # Check if model views were added by checking the labels of menu items
    menu_items = {str(item.name): item for item in admin_app.admin.menu()}
    assert "OneAndTwo" in menu_items  # Category for ModelOne and ModelTwo
    assert "Four" in menu_items  # Category for ModelOne and ModelTwo
    assert "Model One" not in menu_items  # ModelOne should go to a category
    assert "Model Two" not in menu_items  # ModelTwo should go to a category
    assert "Model Three" in menu_items  # ModelThree goes straight to menu
    assert isinstance(menu_items["Model Three"], flask_admin.menu.MenuView)
    assert isinstance(menu_items["OneAndTwo"], flask_admin.menu.MenuCategory)
    assert menu_items["OneAndTwo"].is_category()
    assert not menu_items["Model Three"].is_category()
    submenu_items = {
        str(item.name): item for item in menu_items["OneAndTwo"].get_children()
    }
    assert "Model One" in submenu_items
    assert "Model Two" in submenu_items
    assert not submenu_items["Model One"].is_category()
    assert not submenu_items["Model Two"].is_category()
    assert isinstance(submenu_items["Model One"], flask_admin.menu.MenuView)
    assert isinstance(submenu_items["Model Two"], flask_admin.menu.MenuView)
    four_item = menu_items["Four"].get_children()[0]
    assert four_item.name == "View number Four"
    assert isinstance(four_item, flask_admin.menu.MenuView)


def test_talisman_csp_config_overridden(app):
    """Test that the CSP config of Talisman is set to None."""

    class OldTalisman:
        """Fake Old Talisman class."""

        def __init__(self):
            from werkzeug.local import Local

            self.local_options = Local()
            setattr(
                self.local_options, "content_security_policy", "'default-src': ''self'"
            )

    class NewTalisman:
        """Fake New Talisman class."""

        def __init__(self):
            self.content_security_policy = "'default-src': ''self'"

    class InvenioApp:
        """Fake Invenio App class."""

        def __init__(self, talisman):
            print("ciao")
            self.talisman = talisman

    app.extensions["invenio-app"] = InvenioApp(OldTalisman())
    invenio_app = app.extensions["invenio-app"]
    assert invenio_app.talisman.local_options.content_security_policy

    with app.test_client() as client:
        res = client.get("/login/?user=1")
        assert res.status_code == 200
        res = client.get("/admin/")
        assert res.status_code == 200
        assert not invenio_app.talisman.local_options.content_security_policy

    app.extensions["invenio-app"] = InvenioApp(NewTalisman())
    invenio_app = app.extensions["invenio-app"]
    assert invenio_app.talisman.content_security_policy

    with app.test_client() as client:
        res = client.get("/login/?user=1")
        assert res.status_code == 200
        res = client.get("/admin/")
        assert res.status_code == 200
        assert not invenio_app.talisman.content_security_policy



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Flask-Admin filter utilities."""

from __future__ import absolute_import, print_function

import uuid

from flask_admin.contrib.sqla import filters
from flask_admin.model.filters import convert


class UUIDEqualFilter(filters.FilterEqual):
    """UUID aware filter."""

    def apply(self, query, value, alias):
        """Convert UUID.

        :param query: SQLAlchemy query object.
        :param value: UUID value.
        :param alias: Alias of the column.
        :returns: Filtered query matching the UUID value.
        """
        try:
            value = uuid.UUID(value)
            return query.filter(self.column == value)
        except ValueError:
            return query


class FilterConverter(filters.FilterConverter):
    """Filter converter for dealing with UUIDs and variants."""

    uuid_filters = (UUIDEqualFilter,)

    @convert("uuidtype")
    def conv_uuid(self, column, name, **kwargs):
        """Convert UUID filter."""
        return [f(column, name, **kwargs) for f in self.uuid_filters]

    @convert("variant")
    def conv_variant(self, column, name, **kwargs):
        """Convert variants."""
        return self.convert(str(column.type), column, name, **kwargs)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Flask-Admin form utilities."""

from __future__ import absolute_import, print_function


class LazyChoices(object):
    """Lazy form choices."""

    def __init__(self, func):
        """Initialize lazy choices.

        :param func: Function returning an iterable of choices.
        """
        self._func = func

    def __iter__(self):
        """Iterate over lazy choices."""
        return iter(self._func())



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
# Copyright (C) 2022 RERO.
# Copyright (C) 2023-2024 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio-Admin Flask extension."""

from __future__ import absolute_import, print_function

import warnings

import importlib_metadata
from flask_admin import Admin, AdminIndexView
from flask_login import current_user
from flask_menu import current_menu
from invenio_db import db
from invenio_i18n import lazy_gettext as _
from invenio_theme.proxies import current_theme_icons
from werkzeug.utils import import_string

from . import config
from .proxies import current_admin
from .views import protected_adminview_factory


class _AdminState(object):
    """State for Invenio-Admin."""

    def __init__(self, app, admin, permission_factory, view_class_factory):
        """Initialize state.

        :param app: The Flask application.
        :param admin: The Flask-Admin application.
        :param permission_factory: The permission factory to restrict access.
        :param view_class_factory: The view class factory to initialize them.
        """
        # Create admin instance.
        self.app = app
        self.admin = admin
        self.permission_factory = permission_factory
        self.view_class_factory = view_class_factory

    def register_view(self, view_class, *args, **kwargs):
        """Register an admin view on this admin instance.

        :param view_class: The view class name passed to the view factory.
        :param args: Positional arugments for view class.
        :param kwargs: Keyword arguments to view class.
        """
        protected_view_class = self.view_class_factory(view_class)
        if "endpoint" not in kwargs:
            kwargs["endpoint"] = view_class(*args, **kwargs).endpoint
        self.admin.add_view(protected_view_class(*args, **kwargs))

    def load_entry_point_group(self, entry_point_group):
        """Load administration interface from entry point group.

        :param str entry_point_group: Name of the entry point group.
        """
        for ep in set(importlib_metadata.entry_points(group=entry_point_group)):
            admin_ep = dict(ep.load())
            keys = tuple(k in admin_ep for k in ("model", "modelview", "view_class"))

            if keys == (False, False, True):
                self.register_view(
                    admin_ep.pop("view_class"),
                    *admin_ep.pop("args", []),
                    **admin_ep.pop("kwargs", {}),
                )
            elif keys == (True, True, False):
                warnings.warn(
                    "Usage of model and modelview kwargs are deprecated in "
                    "favor of view_class, args and kwargs.",
                    PendingDeprecationWarning,
                )
                self.register_view(
                    admin_ep.pop("modelview"),
                    admin_ep.pop("model"),
                    admin_ep.pop("session", db.session),
                    **admin_ep,
                )
            else:
                raise Exception(
                    "Admin entry point dictionary must contain "
                    'either "view_class" OR "model" and "modelview" keys.'
                )


class InvenioAdmin(object):
    """Invenio-Admin extension."""

    def __init__(self, app=None, **kwargs):
        """Invenio-Admin extension initialization.

        :param app: The Flask application. (Default: ``None``)
        :param kwargs: Passed to :meth:`init_app`.
        """
        if app:
            self._state = self.init_app(app, **kwargs)

    def init_app(
        self,
        app,
        entry_point_group="invenio_admin.views",
        permission_factory=None,
        view_class_factory=protected_adminview_factory,
        index_view_class=AdminIndexView,
    ):
        """Flask application initialization.

        :param app: The Flask application.
        :param entry_point_group: Name of entry point group to load
            views/models from. (Default: ``'invenio_admin.views'``)
        :param permission_factory: Default permission factory to use when
            protecting an admin view. (Default:
            :func:`~.permissions.admin_permission_factory`)
        :param view_class_factory: Factory for creating admin view classes on
            the fly. Used to protect admin views with authentication and
            authorization. (Default:
            :func:`~.views.protected_adminview_factory`)
        :param index_view_class: Specify administrative interface index page.
            (Default: :class:`flask_admin.base.AdminIndexView`)
        :param kwargs: Passed to :class:`flask_admin.base.Admin`.
        :returns: Extension state.
        """
        self.init_config(app)

        default_permission_factory = app.config["ADMIN_PERMISSION_FACTORY"]
        permission_factory = permission_factory or import_string(
            default_permission_factory
        )

        # Create administration app.

        admin = Admin(
            app,
            name=app.config["ADMIN_APPNAME"],
            template_mode=app.config["ADMIN_TEMPLATE_MODE"],
            index_view=view_class_factory(index_view_class)(),
        )

        # Create admin state
        state = _AdminState(app, admin, permission_factory, view_class_factory)
        if entry_point_group:
            state.load_entry_point_group(entry_point_group)

        app.extensions["invenio-admin"] = state
        return state

    @staticmethod
    def init_config(app):
        """Initialize configuration.

        :param app: The Flask application.
        """
        # Set default configuration
        for k in dir(config):
            if k == "ADMIN_BASE_TEMPLATE" and getattr(config, k) is None:
                continue
            if k.startswith("ADMIN_"):
                app.config.setdefault(k, getattr(config, k))

    def __getattr__(self, name):
        """Proxy to state object.

        :param name: Attribute name of the state.
        """
        return getattr(self._state, name, None)


def finalize_app(app):
    """Finalize app."""
    lazy_base_template(app)
    init_menu(app)


def lazy_base_template(app):
    """Initialize admin base template lazily."""
    base_template = app.config.get("ADMIN_BASE_TEMPLATE")
    invenio_admin = app.extensions["invenio-admin"]
    if base_template:
        invenio_admin.admin.base_template = base_template


def init_menu(app):
    """Initialize menu before first request."""
    # Register settings menu
    current_menu.submenu("settings.admin").register(
        "admin.index",
        # NOTE: Menu item text (icon replaced by a cogs icon).
        _(
            "%(icon)s Administration",
            icon=f'<i class="{current_theme_icons.cogs}"></i>',
        ),  # noqa
        visible_when=_has_admin_access,
        order=100,
    )


def _has_admin_access():
    """Function used to check if a user has any admin access."""
    return (
        current_user.is_authenticated
        and current_admin.permission_factory(current_admin.admin.index_view).can()
    )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Mocks of custom admin views for entrypoint testing."""

from flask_admin.base import BaseView, expose


class Four(BaseView):
    """AdminModelView of the ModelOne."""

    @expose("/")
    def index(self):
        """Index page."""
        return "Content of custom page Four"


four = dict(
    view_class=Four,
    kwargs=dict(
        category="Four",
        name="View number Four",
        endpoint="four",
        menu_icon_type="glyph",
        menu_icon_value="glyphicon-home",
    ),
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Mocks of DB Models and Admin's ModelViews for entrypoint testing."""

from flask_admin.contrib.sqla import ModelView
from invenio_db import db


class ModelOne(db.Model):
    """Test model with just one column."""

    id = db.Column(db.Integer, primary_key=True)
    """Id of the model."""


class ModelTwo(db.Model):
    """Test model with just one column."""

    id = db.Column(db.Integer, primary_key=True)
    """Id of the model."""


class ModelOneModelView(ModelView):
    """AdminModelView of the ModelOne."""

    pass


class ModelTwoModelView(ModelView):
    """AdminModelView of the ModelTwo."""

    pass


# Invalid admin entry point:
zero = {}

# Old deprecated way of specifying admin entry points:
one = dict(modelview=ModelOneModelView, model=ModelOne, category="OneAndTwo")
# New way of specifying admin entry points:
two = dict(
    view_class=ModelTwoModelView,
    args=[ModelTwo, db.session],
    kwargs=dict(category="OneAndTwo"),
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2024 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Administration interface for Invenio applications.

Invenio-Admin is an optional component of Invenio, responsible for registering
and customizing the administration panel for model views and user-defined
admin pages. The module uses standard Flask-Admin features and assumes very
little about other components installed within a given Invenio instance.

Quick start
-----------
This section presents a minimal working example of the Invenio-Admin.

First, let us create a new Flask application:

>>> from flask import Flask
>>> app = Flask('DinerApp')
>>> app.config.update(DB_VERSIONING=False,)

and load the Invenio-DB and Invenio-Admin extensions:

>>> from invenio_db import InvenioDB
>>> from invenio_admin import InvenioAdmin
>>> ext_db = InvenioDB(app)
>>> ext_admin = InvenioAdmin(app, view_class_factory=lambda x: x)

.. warning::

    We use the ``view_class_factory`` parameter above to disable the
    authentication to the admin panel, in order to simplify this tutorial.
    Do not use this for production systems, as you will grant access to the
    admin panel to anonymous users!

    In full application with an authentication system in place, it is
    sufficient to instantiate the extension like:

    ``ext_admin = InvenioAdmin(app)``


Let's now define a simple model with a model view, and one base view:

>>> from invenio_db import db
>>> from flask_admin.contrib.sqla import ModelView
>>> from flask_admin.base import BaseView, expose
>>> class Lunch(db.Model):
...     __tablename__ = 'diner_lunch'
...     id = db.Column(db.Integer, primary_key=True)
...     meal_name = db.Column(db.String(255), nullable=False)
...     is_vegetarian = db.Column(db.Boolean(name='is_v'), default=False)
...
>>> class LunchModelView(ModelView):
...     can_create = True
...     can_edit = True
...
>>> class MenuCard(BaseView):
...     @expose('/')
...     def index(self):
...         return "HelloMenuCard!"

and register them in the admin extension:

>>> ext_admin.register_view(LunchModelView, Lunch, db.session)
>>> ext_admin.register_view(MenuCard)

Finally, initialize the database and run the development server:

>>> from sqlalchemy_utils.functions import create_database
>>> app.config.update(SQLALCHEMY_DATABASE_URI='sqlite:///test.db',
...     SECRET_KEY='SECRET')
...
>>> with app.app_context():
...     create_database(db.engine.url)
...     db.create_all()
>>> app.run() # doctest: +SKIP

You should now be able to access the admin panel `http://localhost:5000/admin
<http://localhost:5000/admin>`_.

Adding admin views from Invenio module
--------------------------------------
In real-world scenarios you will most likley want to add an admin view for
your custom models from within the Invenio module or an Invenio overlay
application. Instead of registering it directly on the application as in the
example above, you can use entry points to register those automatically.

Defining admin views
~~~~~~~~~~~~~~~~~~~~
Let us start with defining the ``admin.py`` file inside your module or overlay,
which will contain all admin-related classes and functions.
For example, assuming a ``Invenio-Diner`` module, the file could reside in:

``invenio-diner/invenio_diner/admin.py``.

In this example we will define two model views for two database models and one
separate base view for statistics page. The content of the file is as follows:

.. code-block:: python

    # invenio-diner/invenio_diner/admin.py
    from flask_admin.base import BaseView, expose
    from flask_admin.contrib.sqla import ModelView
    from invenio_db import db
    from .models import Snack, Breakfast

    class SnackModelView(ModelView):
        can_create = True
        can_edit = True
        can_view_details = True
        column_list = ('id', 'name', 'price', )

    class BreakfastModelView(ModelView):
        can_create = False
        can_edit = False
        can_view_details = True
        column_searchable_list = ('id', 'toast', 'eggs', 'bacon' )

    class DinerStats(BaseView):
        @expose('/')
        def index(self):
            return "Welcome to the Invenio-Diner statistics page!"

        @expose('/sales/')
        def sales(self):
            return "You have served 0 meals!"

    snack_adminview = {
        'view_class': Snack,
        'args': [SnackModelView, db.session],
        'kwargs': {'category': 'Diner'},
    }

    breakfast_adminview = {
        'view_class': Breakfast,
        'args': [BreakfastModelView, db.session],
        'kwargs': {'category': 'Diner'},
    }

    stats_adminview = {
        'view_class': DinerStats,
        'kwargs': {'name': 'Invenio Diner Stats'},
    }

    __version__ = '1.3.2'

    __all__ = (
        'snack_adminview',
        'breakfast_adminview',
        'stats_adminview',
    )

.. note::

    You have to define a dictionary for each BaseView and Model-ModelView pairs
    (see ``stats_adminview``, ``snack_adminview`` and ``breakfast_adminview``
    above) in order to have the admin views automatically registered via
    entry points (see next section).

    The ``args`` and ``kwargs`` keys in the dictionaries are passed to the
    constructor of the view class once it is intialized.

Registering the entry point
~~~~~~~~~~~~~~~~~~~~~~~~~~~
The default way of adding admin views to the admin panel is though
setuptools' entry point discovery. To do that, a newly created module has to
register an entry point under the group ``invenio_admin.views`` inside its
``setup.py`` as follows:

.. code-block:: python

    # invenio-diner/setup.py
    setup(
      entry_points={
        'invenio_admin.views': [
          'invenio_diner_snack = invenio_diner.admin.snack_adminview',
          'invenio_diner_breakfast = invenio_diner.admin.breakfast_adminview',
          'invenio_diner_stats = invenio_diner.admin.stats_adminview',
        ],
      },
    )


Authentication and authorization
--------------------------------
By default Invenio-Admin protects the admin views from unauthenticated users
with Flask-Login and restricts the access on a per-permission basis using
Flask-Security. In order to login to a Invenio-Admin panel the user
needs to be authenticated using Flask-Login and have a Flask-Security
identity which provides the ``ActionNeed('admin-access')``.

.. note::

    If you want to use a custom permission rule, you can easily specify
    your own permission factory in the configuration variable
    :data:`invenio_admin.config.ADMIN_PERMISSION_FACTORY`.

    For more information, see the default factory:
    :func:`invenio_admin.permissions.admin_permission_factory`
    and how the the view is using it:
    :func:`invenio_admin.views.protected_adminview_factory`


Styling
-------
At core, Invenio-Admin uses Flask-Admin for rendering the admin panel
and all of its views. All of the features for defining the ModelViews
and BaseViews can be found in the official Flask-Admin documentation.
Nonetheless, we will mention some of the ones that were already made easy to
use directly in Invenio-Admin.

Custom database type filters
~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Non-basic data types can be made easier to search for and filter using type
filters. This way, fields of certain type that is not searchable by default
can be extended with that functionality. For example see a built-in UUID
filter :class:`invenio_admin.filters.UUIDEqualFilter`. You can enable the
custom fields filters, by setting a variable ``filter_converter`` on the
ModelView class. See an example of a custom filter converter in
:class:`invenio_admin.filters.FilterConverter`.

Assuming that the ``id`` field in ``Snack`` model from the example above is a
UUID-type field, you could enable the UUID filtering on this model as follows:

.. code::

    from invenio_admin.filters import FilterConverter

    class SnackModelView(ModelView):
        filter_converter = FilterConverter()  # Add filter converter
        can_create = True
        can_edit = True
        can_view_details = True
        column_list = ('id', 'name', 'price', )

Base template
~~~~~~~~~~~~~
Styling of the administration interface can be changed via the configuration
variable ``ADMIN_BASE_TEMPLATE``.

If Invenio-Theme is installed,
``ADMIN_BASE_TEMPLATE`` is automatically set to use the
`AdminLTE <https://almsaeedstudio.com/themes/AdminLTE/index2.html>`_ theme
which provides an extra configuration variable ``ADMIN_UI_SKIN`` which controls
the AdminLTE skin (e.g. ``skin-blue`` or ``skin-black``). See AdminLTE
documentation for details on supported skins.

If Invenio-Theme is not installed the default Flask-Admin templates will be
used (based on Bootstrap).

View template mode
~~~~~~~~~~~~~~~~~~
Flask-Admin view templates (forms etc.) can either use Bootstap 2 or 3. By
default the template mode is set to Bootstrap 3 but can be controlled through
``ADMIN_TEMPLATE_MODE`` configuration variable.
"""

from __future__ import absolute_import, print_function

from .ext import InvenioAdmin
from .proxies import current_admin

__version__ = "1.5.1"

__all__ = ("__version__", "InvenioAdmin", "current_admin")



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
# Copyright (C) 2021 Northwestern University.
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Admin view class factory for creating protected admin views on-the-fly."""

from __future__ import absolute_import, print_function

from flask import Blueprint, current_app, redirect, request, url_for
from flask_login import current_user

from .proxies import current_admin

blueprint = Blueprint(
    "invenio_admin",
    __name__,
)


def protected_adminview_factory(base_class):
    """Factory for creating protected admin view classes.

    The factory will ensure that the admin view will check if a user is
    authenticated and has the necessary permissions (as defined by the
    permission factory).
    The factory creates a new class using the provided class as base class
    and overwrites ``is_accessible()`` and ``inaccessible_callback()``
    methods. Super is called for both methods, so the base class can implement
    further restrictions if needed.

    :param base_class: Class to use as base class.
    :type base_class: :class:`flask_admin.base.BaseView`
    :returns: Admin view class which provides authentication and authorization.
    """

    class ProtectedAdminView(base_class):
        """Admin view class protected by authentication."""

        def _handle_view(self, name, **kwargs):
            """Override Talisman CSP header configuration for admin views.

            Flask-Admin extension is not CSP compliant (see:
            https://github.com/flask-admin/flask-admin/issues/1135).
            To avoid UI malfunctions, the CSP header (globally set on each
            request by Talisman extension) must be overridden and removed.
            Remove this code if and when Flask-Admin will be completely CSP
            compliant.
            """
            invenio_app = current_app.extensions.get("invenio-app", None)
            if invenio_app:
                # Because there seems to be back-and-forth on pinning and
                # unpinning dependencies, let's support both for now:
                if hasattr(invenio_app.talisman, "content_security_policy"):
                    invenio_app.talisman.content_security_policy = None
                else:
                    setattr(
                        invenio_app.talisman.local_options,
                        "content_security_policy",
                        None,
                    )
            return super(ProtectedAdminView, self)._handle_view(name, **kwargs)

        def is_accessible(self):
            """Require authentication and authorization."""
            return (
                current_user.is_authenticated
                and current_admin.permission_factory(self).can()
                and super(ProtectedAdminView, self).is_accessible()
            )

        def inaccessible_callback(self, name, **kwargs):
            """Redirect to login if user is not logged in.

            :param name: View function name.
            :param kwargs: Passed to the superclass' `inaccessible_callback`.
            """
            if not current_user.is_authenticated:
                # Redirect to login page if user is not logged in.
                return redirect(
                    url_for(
                        current_app.config["ADMIN_LOGIN_ENDPOINT"], next=request.url
                    )
                )
            super(ProtectedAdminView, self).inaccessible_callback(name, **kwargs)

    return ProtectedAdminView



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Form module tests."""

from __future__ import absolute_import, print_function

from invenio_admin.forms import LazyChoices


def test_lazy_choices():
    """Test lazy choices."""
    called = dict(val=False)

    def _choices():
        called["val"] = True
        return [1, 2]

    choices = LazyChoices(_choices)
    assert not called["val"]
    assert list(choices) == [1, 2]
    assert called["val"]



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Proxy to current extension."""

from __future__ import absolute_import, print_function

from flask import current_app
from werkzeug.local import LocalProxy

current_admin = LocalProxy(lambda: current_app.extensions["invenio-admin"])
